## Technorati
## Copyright 2002 Kalsey Consulting Group
## http://kalsey.com/
## Using this software signifies your acceptance of the license
## agreement that accompanies this software.
##
## Installation and usage instructions can be found at
## http://kalsey.com/tools/technorati/
#
# Last update:
# $Id: Technorati.pm,v 1.10 2003/07/24 19:02:49 Adam Kalsey Exp $

package MTPlugins::Technorati;

use strict;
eval("use MTPlugins::Expressions");

# Set configuration variables
my %config;
$config{key} = "";
$config{cachedir} = "./cache";
$config{nocache} = 0;
$config{debug} = 0;
$config{parser} = '';

my $debug = 0;
my $VERSION = '1.1';

&readconfig;

sub Technorati {
    my($ctx, $args) = @_;
    $ENV{'XML_SIMPLE_PREFERRED_PARSER'} = 'XML::Parser';
    # Set the parser for XML:Simple if the config file requests it.
    $ENV{'XML_SIMPLE_PREFERRED_PARSER'} = $config{parser}
        if (!$config{parser} eq '');
    eval("use XML::Simple");
    return $ctx->error("XML::Simple is not installed. You need to install it in order to run Technorati. See the documentation for instructions on installing the required modules.")
        if ($@);
    use LWP::UserAgent;
	use MT::Log;
	my $log = MT::Log->new;
    my $content = "";
    my($entry, $url, $mode, $links, $details, $content_tree);
    if (exists $args->{debug}) {
        $debug = 1;
    }
    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    # Process expressions inside tag attributes if Expressions is installed.
    # Send a debug warning if it's not.
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    if ($@) {debug("MTPlugins::Expressions is not installed. Expressions will not be processed.");}
    # SWITCH: check for URL and for MT tag searches
    SWITCH: {
            if (exists($args->{url})) {
                $url = $args->{url};
                $mode = "link";
                last SWITCH;
            }
            if (defined($entry = $ctx->stash('entry'))) {
                 my $arch = $ctx->stash('blog')->archive_url;
                $arch .= '/' unless $arch =~ m!/$!;
                $url = $arch . $entry->archive_file($args ? $args->{archive_type} : ());
                $mode = "link";
                last SWITCH;
            }
            # Not in an entry, and no URL specified. Use the blog URL
            $url = $_[0]->stash('blog')->site_url;
            $url .= '/' unless $url =~ m!/$!;
            $mode = "weblog";
    }

    # API Key
    my $devtoken = $config{key}; 
    length($devtoken) > 0
        or return $ctx->error("You need an API Key from Technorati to use this plugin. Visit http://www.technorati.com/members/apikey.html to apply.");

    # Override Retreival method.
    $mode = ($args->{mode}) ? $args->{mode} : $mode;

    debug("Getting $mode for $url");
    my $lastn = $args->{lastn} ? $args->{lastn} : 10;
    $lastn =~ m|^\d*$|
        or return $ctx->error("lastn must be a number.");
    return '' if $lastn == 0;
    # figure how many pages we need to retreive:
    # one page for every 10 or portion thereof
    my $pages = int($lastn/100 + .9999);
    for (my $page = 1; $page <= $pages; $page++) {
        my $start = ($page*100+1)-100;
		my $limit = $lastn <= 100 ? $lastn : 100;
        $content = GetXML($url, $mode, $devtoken, $start, $limit);
        # Get the timestamp for the cache or use Now if the cache is new
        # If there's already a tmestamp tag, use it. Otherwise use the
        # new timestamp
        my $cacheTime = (@{$content}[0]) ? @{$content}[0] : time();
        $cacheTime = $ctx->stash('TechnoratiCacheDateTime') ? 
            $ctx->stash('TechnoratiCacheDateTime') :
            $cacheTime;
        $ctx->stash('TechnoratiCacheDateTime', $cacheTime);
        if (@{$content}[1] eq '') {
            warn('<b>The Technorati service didn\'t answer. It might be down.</b>');
			$log->message("Timeout retrieving data from Technorati.");
			$log->save
				or die $log->errstr;
            return '';
        }
        eval { $content_tree = XMLin(@{$content}[1]) };
		if ($@) {
			$log->message("Technorati error reading XML content: $@.");
			$log->save
				or die $log->errstr;
			return '';
		}
        next unless (!$@ and ref $content_tree and $content_tree->{document}->{item});
        # Load the links from the current iteration of the XML doc
        if ((ref $content_tree->{document}->{item}) eq 'HASH') {
            $details = [ $content_tree->{document}->{item} ];
        } else {
            $details = $content_tree->{document}->{item};
        }
        # Load the curent iteration's links into the main array
        push @$links, @$details;
    }
    return '' unless defined @$links; 
    my $count;
    my $link = '';
    for my $i (@$links) {
        last if ++$count > $lastn;
        $ctx->stash('TechnoratiLink', $i);
        my $out = $builder->build($ctx, $tokens);
        return $ctx->error( $builder->errstr ) unless defined $out;
        $link .= $out;
    }
    $link;
}

sub TechnoratiLinkTitle {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{weblog}->{name} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkURL {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{weblog}->{url} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkRSS {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{weblog}->{rssurl} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkInbound {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{weblog}->{inboundlinks} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkBlogs {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{weblog}->{inboundblogs} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkPermalink {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{nearestpermalink} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkExcerpt {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('TechnoratiLink'))
        or return '';
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    $i->{excerpt} || $args->{'if_blank'} || '';
}

sub TechnoratiLinkCreated { 
    # technorati dates come in as
    # yyyy-mm-dd hh:mm:ss GMT
    my($ctx, $args) = @_; 
    # Process expressions inside tag attributes if Expressions is installed.
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    defined(my $i = $ctx->stash('TechnoratiLink')) 
        or return ''; 
    use MT::Util qw( format_ts ); 
    return '' 
        unless $i->{linkcreated}; 
    return $i->{linkcreated} unless $args->{'format'}; 
    my $date = $i->{linkcreated}; 
    # The technorati date format has all the elements in the same order as
    # an MT datestamp, but includes formatting in order to be human-readable.
    $date =~ s/\D//g; # strip non-numbers out
    my $out = format_ts($args->{'format'}, $date, $ctx->stash('blog')); 
    $out; 
}

sub CacheDateTime {
    my($ctx, $args) = @_;
    # Process expressions inside tag attributes if Expressions is installed.
    eval{$args = MTPlugins::Expressions::process($ctx, $args)};
    use MT::Util qw( format_ts );
    my @ts = localtime($ctx->stash('TechnoratiCacheDateTime'));
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $ts[5]+1900, $ts[4]+1, @ts[3,2,1,0];
    my $format = $args->{'format'} ? $args->{'format'} : "%B %e, %Y %I:%M %p";
    my $out = format_ts($format, $ts, $ctx->stash('blog'));
    $out;
}

 sub Version {
     $VERSION;
 }

# ---- Utility Functions --------

sub getRemoteXML {
    my($url, $mode, $devtoken, $start, $limit) = @_;
    my $APIurl = "http://api.technorati.com/cosmos?" .
        "key=$devtoken&" .
        "url=$url&" .
        "type=$mode&start=$start&limit=$limit";
    debug("Getting $APIurl");
    my $ua = new LWP::UserAgent;
    $ua->agent("Technorati MT Plugin/$VERSION");
    my $http_request = new HTTP::Request('GET', $APIurl);
    my $http_response = $ua->request($http_request);
    my $content = $http_response->{'_content'};
    $content =~ s/<[^\/]([^>]+)>\s+<\/[^>]+>/<$1 \/>/g; # convert nodes that contain only spaces to empty nodes
    $content =~ s/&(?!#?[xX]?(?:[0-9a-fA-F]+|\w{1,8});)/&amp;/g; #escape ampersands
    $content;
}

sub SaveFile {
    my($content, $file) = @_;
    return unless ($content ne '');
    if ($config{cachedir} && !$config{nocache}) {
        debug("Saving cache to $file");
        open (CACHE, ">$file") or warn "Can't write to $file: $!\n";
        print CACHE "$content";
        close CACHE;
    }
}

sub GetXML {
    my($url, $mode, $devtoken, $start, $limit) = @_;
    my @content;
    my $file = "$url.$mode";
    $file =~ s/(\W|%20)//g;
    $file = $config{cachedir}."/technorati." . $file . $limit . ".$start.xml";
    if ($config{'nocache'}) {
        debug("Caching is disabled.");
        $content[0] = 0;
        $content[1] = getRemoteXML($url, $mode, $devtoken, $start, $limit);
        return \@content;
    }
    open (CACHE, "<$file");
    my @stat = stat($file) unless eof(CACHE);
    close CACHE;
    if (@stat) {
        # Cache exists
        debug("Cache Exists: $file");
        my $CacheTTL = time() - $stat[9];
        if ($CacheTTL > 3600) {
            # Cache is stale
            debug("The cache is stale");
            $content[1] = getRemoteXML($url, $mode, $devtoken, $start, $limit);
            SaveFile($content[1], $file);
            return \@content;
        }
    } else {
        # Cache doesn't exist
        debug("No cache for this query");
        $content[0] = 0;
        $content[1] = getRemoteXML($url, $mode, $devtoken, $start, $limit);
        SaveFile($content[1], $file);
        return \@content;
    }
    # if we got here, then the cache existed and was fresh
    # so load from cache
    $content[0] = $stat[9];
    $content[1] = $file;
    \@content;
}

# Print troubleshooting messages if the debug flag is set in the
# template or config file
sub debug {
    my $msg = shift;
    warn "$msg\n" if $debug or $config{debug};
}

# Read technorati.config from the MT directory and add the 
# config parameters to the plugin.
sub readconfig {
    my($path);
    if ($0 =~ m!(.*[/\\])!) {
        $path = $1;
    } else {
        $path = './';
    }
    my $cfgfile = File::Spec->catfile($path,'technorati.config');
    open FH, $cfgfile or
        die("Error opening file '$cfgfile': $!");
    while (<FH>) {
        chomp;
        next if !/\S/ || /^#/;
        my($var, $val) = $_ =~ /^\s*(\S+)\s+(.+)$/;
        $var = lc($var);
        $val =~ s/\s*$//;
        next unless $var && $val;
        next unless exists $config{$var};
        # If the value is "true" or "false" set it to 1 or 0 respectively
        if (lc($val) eq 'false') {
            $val = 0;
        } elsif(lc($val) eq 'true') {
            $val = 1;
        }
        $config{$var} = $val;
    }
    close FH;
    1;
}

1;